/*
 * PmodPMON1.c
 *
 *  Created on: Feb 26, 2018
 *      Author: Krzysztof Chojnowski
 */

#include <PmodPMON1.h>
#include "stm32l4xx_hal.h"

// The address of the potentiometer
#define PMODPMON1_POT_ADDRES 0x5E

// The address of the monitor
#define PMODPMON1_MON_ADDRES 0x60

I2C_HandleTypeDef pmodPmon1I2c;

void PmodPMON1_Config(void)
{
  __HAL_RCC_GPIOF_CLK_ENABLE();
  GPIO_InitTypeDef GPIO_InitStruct;

  // Configure the PF14 pin to handle CONV input
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_VERY_HIGH;
  GPIO_InitStruct.Pull = GPIO_PULLUP;
  GPIO_InitStruct.Pin = GPIO_PIN_14;
  HAL_GPIO_Init(GPIOF, &GPIO_InitStruct);

  // Enable the continuous conversion by keeping the CONV pin high.
  HAL_GPIO_WritePin(GPIOF, GPIO_PIN_14, GPIO_PIN_SET);

  // Configure the interrupt (PF15) for alert.
  GPIO_InitStruct.Mode = GPIO_MODE_IT_RISING_FALLING;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_VERY_HIGH;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  GPIO_InitStruct.Pin = GPIO_PIN_15;
  HAL_GPIO_Init(GPIOF, &GPIO_InitStruct);

  HAL_NVIC_SetPriority(EXTI15_10_IRQn, 2, 0);
  HAL_NVIC_EnableIRQ(EXTI15_10_IRQn);

  pmodPmon1I2c.Instance = I2C2;
  pmodPmon1I2c.Init.AddressingMode = I2C_ADDRESSINGMODE_7BIT;
  pmodPmon1I2c.Init.DualAddressMode = I2C_DUALADDRESS_DISABLE;
  pmodPmon1I2c.Init.GeneralCallMode = I2C_GENERALCALL_DISABLE;
  pmodPmon1I2c.Init.NoStretchMode = I2C_NOSTRETCH_DISABLE;
  pmodPmon1I2c.Init.OwnAddress1 = 0x01;

  // Set the I2C_TIMINGR register.
  pmodPmon1I2c.Init.Timing = 0x10563046;

  HAL_I2C_Init(&pmodPmon1I2c);

  PmodMON1_ClearAlarm();

  // Enable the continuous current and voltage measurement in the ALERT_EN register.
  uint8_t monCommand = 0x05;
  HAL_I2C_Master_Transmit(&pmodPmon1I2c, PMODPMON1_MON_ADDRES, &monCommand, 1, 100);

  // The potentiometer configuration sets the overcurrent alarm level. In this example
  // the potentiometer is set to 1/64 * RAB + RW ~= 140 Ohm. For the 3V supply, this
  // configuration gives ~= 89 mV. Considering the current sense amplifier gain and
  // the sense resistor value (0.1 Ohm), The alarm will be triggered for the measured
  // current greater than 49 mA.
  uint8_t potCommand[2] = {0x02, 0x01};
  HAL_I2C_Master_Transmit(&pmodPmon1I2c, PMODPMON1_POT_ADDRES, potCommand, 2, 100);
}

void HAL_I2C_MspInit(I2C_HandleTypeDef *hi2c)
{
  __HAL_RCC_I2C2_CLK_ENABLE();
  __HAL_RCC_GPIOF_CLK_ENABLE();

  GPIO_InitTypeDef GPIO_InitStruct;

  GPIO_InitStruct.Mode = GPIO_MODE_AF_OD;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_VERY_HIGH;
  GPIO_InitStruct.Alternate = GPIO_AF4_I2C2;
  GPIO_InitStruct.Pin = GPIO_PIN_0 | GPIO_PIN_1;
  HAL_GPIO_Init(GPIOF, &GPIO_InitStruct);
}

void PmodPMON1_GetMeasurements(uint16_t* voltage, uint16_t* current)
{
  // Read the current and voltage measurements.
  uint8_t data[3] = {0};
  HAL_I2C_Master_Receive(&pmodPmon1I2c, PMODPMON1_MON_ADDRES, data, 3, 100);

  // The conversion equations are taken from the ADM1191 Data Sheet
  // Voltage = (V_FULLSCALE / 4096) × Code); V_FULLSCALE has the default value 26.52 V
  *voltage = 25620 * ((data[0] << 4) | (data[2] >> 4)) / 4096;

  // Current = ((I_FULLSCALE / 4096) × Code) / Sense Resistor;
  // I_FULLSCALE = 105.84 mV; Sense Resistor = 0.1 Ohm.
  *current = 105.84 * ((data[1] << 4) | (data[2] & 0x0F)) / (4096 * 0.1);
}

void PmodMON1_ClearAlarm(void)
{
  // Clear the alarm by setting the Clear bit in the ALERT_EN register.
  uint8_t monCommand[2] = {0x81, 0x14};
  HAL_I2C_Master_Transmit(&pmodPmon1I2c, PMODPMON1_MON_ADDRES, monCommand, 2, 100);
}
