/*
******************************************************************************
File:     main.c
Info:     Generated by Atollic TrueSTUDIO(R) 8.0.0   2017-10-28

The MIT License (MIT)
Copyright (c) 2009-2017 Atollic AB

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

******************************************************************************
*/

#include <PmodPMON1.h>
#include <string.h>

#include "stm32l4xx.h"
#include "stm32l4xx_hal.h"
#include "system_clock.h"
#include "serial.h"
#include "led.h"

uint8_t alarmFlag = 0;

int main(void)
{
  HAL_Init();
  SystemClock_Config();
  Serial_Config();
  Led_Config();

  PmodPMON1_Config();

  char textBuffer[128];   // Text buffer for serial writing.

  uint16_t voltage = 0;
  uint16_t current = 0;

  while(1) {
    HAL_Delay(1000);

    // If the alarm flag is set clear the alarm.
    if(alarmFlag == 1) {
      PmodMON1_ClearAlarm();
    }

    // Read the voltage and current measurements
    PmodPMON1_GetMeasurements(&voltage, &current);

    // Write the obtained values to the serial port.
    sprintf(textBuffer, "Voltage: %dmV Current %dmA\n", voltage, current);
    Serial_Write(textBuffer, strlen(textBuffer));
  }
}

/**
 * @brief This function handles the interrupts configured in the PmodPMON1 driver.
 *        In the example there is only single interrupt source active so checking
 *        its source is not necessary.
 */
void HAL_GPIO_EXTI_Callback(uint16_t GPIO_Pin)
{
  // The alarm is active when the pin is in the low level.
  // In case of alarm turn on the LED and set the flag.
  if(HAL_GPIO_ReadPin(GPIOF, GPIO_PIN_15) == GPIO_PIN_RESET) {
	Led_TurnOn(LED1);
    alarmFlag = 1;
  } else {
    Led_TurnOff(LED1);
    alarmFlag = 0;
  }
}
